#! /usr/bin/perl

#AUTHOR
#  Adel Dayarian

#The code is available freely, under the GNU Public License, at http://www.physics.rutgers.edu/~anirvans/SOPRA/

use strict;
use Getopt::Long;

my @fragment_names;
my @mate_names;
my @distances;
my $address="";

&GetOptions( "frag=s{,}" => \@fragment_names,
             "mate=s",  => \@mate_names,
     	     "d=i",  => \@distances,
     	     "a=s", => \$address,
           ) or die "Unknown option\n";  
#-------------------------------------------------   
if( ( (! @mate_names) && (! @fragment_names) ) || ( $address eq "" ) ){
   print "Usage: $0\n\n";
   print "-mate  File in fasta format containing reads for a paired-end library, has to be followed by option -d\n";
   print "-d  The insert size for the corresponding paired-end library\n\n"; 
   print "-frag  File in fasta format containing reads for a fragment (unpaired) library\n\n";
   print "-a  Name of the output directory\n\n";   
   
   print "example: $0  -mate mate1.fasta -d L1 -mate mate2.fasta -d L2 -frag fragment1.fasta fragment2.fasta -a mydir_sout\n\n";
   exit;
}

if( $address =~ /^(.*)\/$/ ){
     $address=$1; 
}

print "Fragment files:\n@fragment_names\n";
print "Paired files:\n@mate_names\n";
print "Inser sizes:\n@distances\n";
print "Output directory:\n$address\n";

if ( scalar(@mate_names)==0 ){
   print "\nNo mate-pair file specified\n";
}

if ( ( scalar(@mate_names) > 0 ) && ( scalar(@mate_names)!=scalar(@distances) ) ){
   die "You specified that your input consist of mate pairs, but have not specified valid insert length (use -d option)\n";
}


if ( ($address ne "." ) && ($address ne "" ) ){     
	 mkdir("$address", 0777) || print $!;
	 $address=$address."/sread_in.fasta";
}else{
	 $address=$address."/sread_in.fasta";
}

open (OUT,">$address"); 

my $tot_seq; my $num_seq;
my $num_matefile=scalar(@mate_names)-1;

for (my $i=0; $i <= $num_matefile ; $i++){
    $num_seq=0;
    open (MATE_FILE,"<$mate_names[$i]") or die "Cannot open $mate_names[$i] $!\n";
    
    if( $distances[$i] > 0 ){
        print OUT "Mate.$mate_names[$i]"."_$distances[$i]\n";
    }else{
        die "Invalid insert length $distances[$i]\n";
    }

   my $avali=1; my $start=0; my ($ori_header,$short_r); 

    while (my $line=<MATE_FILE>){
    	chomp($line);
        if( $line =~ /^\#/ ){
        
        }elsif( $line =~ /^\>/ ){
           
           if($start==1){        
			   if ( $avali==1 ){
					print OUT "$short_r\n"; $tot_seq++; $num_seq++;
			   }else{
					print OUT "$ori_header\n";
					print OUT "$short_r:"; $tot_seq++; $num_seq++;
			   }         
           	   $short_r=''; 
           }else{
           	   $start=1;
           }
           if ( $avali==1 ){
           		$ori_header=$line;
           		$avali=0;
           }else{
           		$avali=1;
           }           
        }else{
           $short_r.=$line;	     
        }
    }   
	print OUT "$short_r\n"; $tot_seq++; $num_seq++;
	close MATE_FILE; 
	
    print "\n$mate_names[$i]        number of reads: $num_seq          insert length: $distances[$i]\n";
}


my $num_fragmentfile=scalar(@fragment_names)-1;

for (my $i=0; $i <= $num_fragmentfile ; $i++){
    $num_seq=0;
    open (FRAG,"<$fragment_names[$i]") or die "Cannot open $fragment_names[$i] $!\n";

    print OUT "Fragment\n";
    
    while (my $line=<FRAG>){
        if( $line =~ /^\#/ ){
        
        }elsif( $line =~ /^\>/ ){
            print OUT "$line";
            $line=<FRAG>; print OUT "$line";

			$tot_seq++;	$num_seq++;
			
        }else{
            die "$fragment_names[$i] is not in the right format:\n$line\n";
        }
    }
    close FRAG; 
    print "$fragment_names[$i]      number of reads: $num_seq\n";
}
close OUT;

print "\nTotal number of reads  $tot_seq\n\n";